# Copyright 2016 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient.common import extension


class ExtLink(extension.NeutronClientExtension):
    resource = 'ext_link'
    resource_plural = 'ext_links'
    path = 'ext-links'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


class ExtLinkList(extension.ClientExtensionList,
                  ExtLink):
    """List connectors/links that belong to a given tenant."""

    shell_command = 'ext-link-list'
    list_columns = ['id', 'name', 'local_tenant', 'remote_tenant',
                    'local_ne_type', 'local_ne_id',
                    'remote_ne_type', 'remote_ne_id']
    pagination_support = True
    sorting_support = True
